"""Core module for defining global settings of the Vantage Agent."""

import sys
from pathlib import Path
from typing import Annotated, Optional

from pydantic import AnyHttpUrl, Field, field_validator
from pydantic_settings import BaseSettings, SettingsConfigDict

from vantage_agent.logging import logger


class Settings(BaseSettings):
    """Settings for the Vantage Agent."""

    SCONTROL_PATH: Path = Path("/usr/bin/scontrol")

    # Vantage API info
    BASE_API_URL: Annotated[str, AnyHttpUrl] = "https://apis.vantagehpc.io"

    # Sentry
    SENTRY_DSN: Optional[AnyHttpUrl] = None
    SENTRY_ENV: str = "local"

    # OIDC config for machine-to-machine security
    OIDC_DOMAIN: str
    OIDC_CLIENT_ID: str
    OIDC_CLIENT_SECRET: str
    OIDC_USE_HTTPS: bool = True

    CACHE_DIR: Path = Path.home() / ".cache/vantage-agent"

    # Task settings
    TASK_JOBS_INTERVAL_SECONDS: int = Field(30, ge=10, le=3600)  # seconds
    TASK_SELF_UPDATE_INTERVAL_SECONDS: Optional[int] = Field(30, ge=10)  # seconds

    @field_validator("SCONTROL_PATH", mode="after")
    @classmethod
    def validate_scontrol_path(cls, v: Path) -> Path:
        """Ensure that the SCONTROL_PATH is an absolute path."""
        if not v.is_absolute():
            raise ValueError("SCONTROL_PATH must be an absolute path")
        return v

    model_config = SettingsConfigDict(env_prefix="VANTAGE_AGENT_", env_file=".env", extra="ignore")


def _init_settings() -> Settings:
    try:
        return Settings()  # type: ignore[call-arg]
    except ValueError as e:
        logger.error(e)
        sys.exit(1)


SETTINGS = _init_settings()



"""Core module for agent health related operations.

Copied and adjusted from https://github.com/omnivector-solutions/jobbergate/blob/main/jobbergate-agent/jobbergate_agent/jobbergate/report_health.py.
"""

from textwrap import dedent

from loguru import logger

from vantage_agent import lock
from vantage_agent.exceptions import VantageApiError
from vantage_agent.logging import log_error
from vantage_agent.settings import SETTINGS
from vantage_agent.vantage_api_client import backend_client


async def report_health_status() -> None:
    """Ping the API to report the agent's status."""
    with lock:
        logger.debug("Reporting status to the API")
        with VantageApiError.handle_errors("Failed to report agent status", do_except=log_error):
            query = dedent(
                """\
                mutation ($clientId: String!, $interval: Int!) {
                    reportAgentHealth(clientId: $clientId, interval: $interval)
                }"""
            )
            body = {
                "query": query,
                "variables": {
                    "clientId": SETTINGS.OIDC_CLIENT_ID,
                    "interval": SETTINGS.TASK_JOBS_INTERVAL_SECONDS,
                },
            }
            response = await backend_client.post("cluster/graphql", json=body)
            response.raise_for_status()



"""Core module for defining operations related to the scontrol command."""

import asyncio
import subprocess

from jsondiff import diff
from loguru import logger

from vantage_agent import lock
from vantage_agent.helpers import (
    load_cached_dict,
    parse_slurm_config,
    parse_slurm_nodes,
    parse_slurm_partitions,
)
from vantage_agent.logicals.vantage_api import (
    upload_slurm_config,
    upload_slurm_nodes,
    upload_slurm_partitions,
)
from vantage_agent.settings import SETTINGS


def upload_scontrol_config():
    """Upload Slurm's configuration to the Vantage API."""
    with lock:
        logger.debug("Getting Slurm's configuration")
        result = subprocess.run([SETTINGS.SCONTROL_PATH, "show", "config"], stdout=subprocess.PIPE, text=True)
        output = result.stdout

        logger.debug("Parsing Slurm's configuration")
        current_config = parse_slurm_config(output)

        logger.debug("Loading cached Slurm's configuration")
        cached_config = load_cached_dict("slurm_config.json")

        logger.debug("Computing difference between cached and current configurations")
        diff_config = diff(cached_config, current_config, marshal=True)

        if diff_config:
            logger.debug("Detected changes in Slurm's configuration. Uploading to the API...")
            asyncio.run(upload_slurm_config(diff_config))
        else:
            logger.debug("No changes detected in Slurm's configuration.")


def upload_scontrol_partition():
    """Upload Slurm's partitions information to the Vantage API."""
    with lock:
        logger.debug("Getting Slurm's partition information")
        result = subprocess.run(
            [SETTINGS.SCONTROL_PATH, "show", "partition"], stdout=subprocess.PIPE, text=True
        )
        output = result.stdout

        logger.debug("Parsing Slurm's partitions")
        current_config = parse_slurm_partitions(output)

        logger.debug("Loading cached Slurm's partitions")
        cached_config = load_cached_dict("slurm_partitions.json")

        logger.debug("Computing difference between cached and current partitions")
        diff_partitions = diff(cached_config, current_config, marshal=True)

        if diff_partitions:
            logger.debug("Detected changes in Slurm's configuration. Uploading to the API...")
            asyncio.run(upload_slurm_partitions(diff_partitions))
        else:
            logger.debug("No changes detected in Slurm's partitions.")


def upload_scontrol_node():
    """Upload Slurm's nodes information to the Vantage API."""
    with lock:
        logger.debug("Getting Slurm's node information")
        result = subprocess.run([SETTINGS.SCONTROL_PATH, "show", "node"], stdout=subprocess.PIPE, text=True)
        output = result.stdout

        logger.debug("Parsing Slurm's nodes")
        current_config = parse_slurm_nodes(output)

        logger.debug("Loading cached Slurm's nodes")
        cached_config = load_cached_dict("slurm_nodes.json")

        logger.debug("Computing difference between cached and current nodes")
        diff_nodes = diff(cached_config, current_config, marshal=True)

        if diff_nodes:
            logger.debug("Detected changes in Slurm's nodes. Uploading to the API...")
            asyncio.run(upload_slurm_nodes(diff_nodes))
        else:
            logger.debug("No changes detected in Slurm's nodes.")
        pass
